@@ -35,7 +35,7 @@ import re
 
 PY2 = sys.version_info[0] == 2
 
-VERSION = (0, 2, 0)
+VERSION = (0, 2, 1)
 __version__ = '%s.%s.%s' % VERSION
 apilevel = '2.0'
 threadsafety = 1
@@ -348,13 +348,16 @@ def escape_parameter(v):
     if v is None:
         return 'NULL'
     elif (PY2 and t == unicode) or (not PY2 and t == str):  # string
+        return u"'" + v.replace(u"'", u"''") + u"'"
+    elif PY2 and t == str:    # PY2 str
+        v = ''.join(['\\%03o' % (ord(c), ) if ord(c) < 32 or ord(c) > 127 else c for c in v])
         return "'" + v.replace("'", "''") + "'"
-    elif PY2 and t == str:  # binary
-        return "'" + ''.join(['\\%03o' % (ord(c),) for c in v]) + "'::bytea"
-    elif t == bytes:        # binary
+    elif t == bytearray or t == bytes:        # binary
         return "'" + ''.join(['\\%03o' % (c, ) for c in v]) + "'::bytea"
     elif t == bool:
         return u"'t'" if v else u"'f'"
+    elif t == int or t == float or t == long:
+        return str(v)
     else:
         return "'" + str(v) + "'"
 
@@ -364,6 +367,7 @@ class Cursor(object):
         self.description = []
         self._rows = []
         self._current_row = -1
+        self.rowcount = 0
 
     def execute(self, query, args=()):
         DEBUG_OUTPUT('Cursor::execute()', query, args)
@@ -374,6 +378,14 @@ class Cursor(object):
         self.args = args
         self.connection.execute(self, query, args)
 
+    def executemany(self, query, seq_of_params):
+        DEBUG_OUTPUT('Cursor::executemany()', query, seq_of_params)
+        rowcount = 0
+        for params in seq_of_params:
+            self.execute(query, params)
+            rowcount += self.rowcount
+        self.rowcount = rowcount
+
     def fetchone(self):
         self._current_row += 1
         DEBUG_OUTPUT('Cursor::fetchone()', self._current_row, self.rowcount)
@@ -390,10 +402,6 @@ class Cursor(object):
     def fetchall(self):
         return self._rows
 
-    @property
-    def rowcount(self):
-        return len(self._rows)
-
     def close(self):
         self.connection = None
 
@@ -463,6 +471,11 @@ class Connection(object):
             elif code == PG_B_BACKEND_KEY_DATA:
                 DEBUG_OUTPUT("BACKEND_KEY_DATA:", binascii.b2a_hex(data))
             elif code == PG_B_COMMAND_COMPLETE:
+                command = data[:-1].decode('ascii')
+                for k in ('SELECT', 'UPDATE', 'DELETE', 'INSERT'):
+                    if command[:len(k)] == k:
+                        obj.rowcount = int(data[:-1].decode('ascii').split(' ')[-1])
+                        obj._current_row = -1
                 DEBUG_OUTPUT("COMMAND_COMPLETE:", data[:-1].decode('ascii'))
             elif code == PG_B_ROW_DESCRIPTION:
                 DEBUG_OUTPUT("ROW_DESCRIPTION:", binascii.b2a_hex(data))
@@ -506,7 +519,7 @@ class Connection(object):
                 for i in range(len(row)):
                     row[i] = _decode_column(row[i], obj.description[i][1], self.encoding)
 
-                obj._rows.append(row)
+                obj._rows.append(tuple(row))
                 DEBUG_OUTPUT("\t\t", row)
             elif code == PG_B_NOTICE_RESPONSE:
                 DEBUG_OUTPUT("NOTICE_RESPONSE:", binascii.b2a_hex(data))
@@ -587,6 +600,9 @@ class Connection(object):
         self._cursor = self.cursor()
         self._process_messages()
 
+    def _is_connect(self):
+        return bool(self.sock)
+
     def _close(self):
         self.sock.close()
         self.sock = None
@@ -597,6 +613,7 @@ class Connection(object):
     def execute(self, obj, query, args=()):
         if args:
             escaped_args = tuple(escape_parameter(arg) for arg in args)
+            query = query.replace('%', '%%').replace('%%s', '%s')
             query = query % escaped_args
         if not self.in_transaction:
             self._send_message(PG_F_QUERY, b"BEGIN\x00")
@@ -615,18 +632,22 @@ class Connection(object):
         self.autocommit = autocommit
 
     def begin(self):
+        DEBUG_OUTPUT('BEGIN')
         self._send_message(PG_F_QUERY, b"BEGIN\x00")
         self._process_messages(self._cursor)
 
     def commit(self):
+        DEBUG_OUTPUT('COMMIT')
         self._send_message(PG_F_QUERY, b"COMMIT\x00")
         self._process_messages(self._cursor)
         self.begin()
 
     def rollback(self):
-        self._send_message(PG_F_QUERY, b"ROLLBACK\x00")
-        self._process_messages(self._cursor)
-        self.begin()
+        DEBUG_OUTPUT('ROLLBACK')
+        if self.sock:
+            self._send_message(PG_F_QUERY, b"ROLLBACK\x00")
+            self._process_messages(self._cursor)
+            self.begin()
 
     def reopen(self):
         self.close()
